﻿namespace PopupControlContainerPlayground
{
	#region Enumerations

	enum EControlState
	{
		Closed,
		Opening,
		Opened,
		Closing
	}

	#endregion

	internal class ControlState
	{
		private readonly EControlState _controlState = EControlState.Closed;

		public static readonly ControlState Closed = new ControlState(EControlState.Closed);
		public static readonly ControlState Opening = new ControlState(EControlState.Opening);
		public static readonly ControlState Opened = new ControlState(EControlState.Opened);
		public static readonly ControlState Closing = new ControlState(EControlState.Closing);

		public static bool operator ==(ControlState left, ControlState right)
		{
			// If both are null, or both are same instance, return true
			if (ReferenceEquals(left, right))
			{
				return true;
			}

			// If one is null, but not both, return false
			if (((object)left == null) || ((object)right == null))
			{
				return false;
			}

			// Return true if the control state match
			return left._controlState == right._controlState;
		}

		public static bool operator !=(ControlState left, ControlState right)
		{
			return !(left == right);
		}

		public ControlState()
		{
		}

		private ControlState(EControlState controlState)
		{
			_controlState = controlState;
		}

		public override string ToString()
		{
			return _controlState.ToString();
		}

		private bool Equals(ControlState other)
		{
			if (ReferenceEquals(null, other)) return false;
			if (ReferenceEquals(this, other)) return true;
			return Equals(other._controlState, _controlState);
		}

		public override bool Equals(object obj)
		{
			if (ReferenceEquals(null, obj)) return false;
			if (ReferenceEquals(this, obj)) return true;
			if (obj.GetType() != typeof(ControlState)) return false;
			return Equals((ControlState)obj);
		}

		public override int GetHashCode()
		{
			return _controlState.GetHashCode();
		}
	}
}
